/*---------------------------------------------------------------------------------
	GplFlash port for DS. Please see COPYING for GPL information.
	Ported by natrium42.

	Based on template.c by WinterMute from devkitPro.
---------------------------------------------------------------------------------*/

#include "nds.h"
#include <nds/arm9/console.h> //basic print funcionality
#include <stdio.h>
#include <stdlib.h>
#include <flash.h>

#include <malloc.h>
#include <sys/time.h>

#include "swf_bin.h"

u16 framebuf[256*192];
FlashHandle flashHandle;
struct FlashInfo fi;
FlashDisplay fd;
struct timeval wd;
bool playing = false;

volatile int frame = 0;

//---------------------------------------------------------------------------------
void Vblank() {
//---------------------------------------------------------------------------------
	frame++;

	if(!playing) return; // RETURN!!!

	FlashEvent fe;
	touchPosition touchXY = touchReadXY();

	scanKeys();

	if(keysDown() & KEY_TOUCH) {
		fe.type = FeButtonPress;
		FlashExec(flashHandle,FLASH_EVENT,&fe,&wd);
	} else if (keysUp() & KEY_TOUCH) {
		fe.type = FeButtonRelease;
		FlashExec(flashHandle,FLASH_EVENT,&fe,&wd);
	} else if (keysHeld() & KEY_TOUCH) {
		fe.type = FeMouseMove;
		fe.x = touchXY.px;
		fe.y = touchXY.py;
		FlashExec(flashHandle,FLASH_EVENT,&fe,&wd);
	}
}

//---------------------------------------------------------------------------------
int main(void) {
//---------------------------------------------------------------------------------
	irqInit();
	irqSet(IRQ_VBLANK, Vblank);
	irqEnable(IRQ_VBLANK);


	vramSetMainBanks(VRAM_A_LCD, VRAM_B_LCD, VRAM_C_SUB_BG, VRAM_D_SUB_SPRITE);

	// Turn on the screens and 2D cores and switch to mode 0
	POWER_CR = POWER_ALL_2D;
	videoSetMode(MODE_FB0);

	videoSetModeSub(MODE_0_2D | DISPLAY_BG0_ACTIVE);	//sub bg 0 will be used to print text
	vramSetBankC(VRAM_C_SUB_BG);


	SUB_BG0_CR = BG_MAP_BASE(31);
	BG_PALETTE_SUB[255] = RGB15(31,31,31);	//by default font will be rendered with color 255
	consoleInitDefault((u16*)SCREEN_BASE_BLOCK_SUB(31), (u16*)CHAR_BASE_BLOCK_SUB(0), 16);

	iprintf("GPLFlash\n");





	flashHandle = FlashNew();

	if (flashHandle == 0) {
		iprintf("Couldn't create handle!\n");
	} else {
		iprintf("Created handle!\n");
	}

	iprintf("Filesize: %i bytes\n", swf_bin_size);

	iprintf("Parsing flash... ");
	// Load level 0 movie
	int status;
	int steps = 0;
	do {
		status = FlashParse(flashHandle, 0, (char*) swf_bin, (int) swf_bin_size);
		steps++;
	} while (status & FLASH_PARSE_NEED_DATA);
	iprintf("Done (%i steps)\n", steps);
	iprintf("Status: %i\n", status);

	FlashGetInfo(flashHandle, &fi);
	iprintf("Frame: %i x %i\n", fi.frameWidth, fi.frameHeight);
	iprintf("Version: %i\n", fi.version);
	iprintf("Frame count: %i\n", fi.frameCount);

	fd.width = 256;
	fd.height = 192;
	fd.bpl = 256 * 2;
	fd.depth = 16;
	fd.bpp = 2;
	fd.clip_x = 0;
	fd.clip_y = 0;
	fd.clip_width = 256;
	fd.clip_height = 192;
	fd.pixels = framebuf;
	iprintf("Init gfx\n");
	status = FlashGraphicInit(flashHandle, &fd);
	iprintf("FlashGraphicInit() = %i\n", status);
	iprintf("Done\n");

	FlashSoundInit(flashHandle, "/dev/dsp"); // heh
	FlashSettings(flashHandle, PLAYER_LOOP);

	playing = true;



	long cmd, wakeUp;

	cmd = FLASH_REWIND;
	FlashExec(flashHandle, cmd, 0, &wd);
	cmd = FLASH_CONT;
	wakeUp = FlashExec(flashHandle, cmd, 0, &wd);

	//touchPosition touchXY;

	while (1) {
		// write touch coordinates (to test that we haven't crashed...)
		//touchXY=touchReadXY();
		//iprintf("\x1b[10;0HTouch x = %04X, %04X\n", touchXY.x, touchXY.px);
		//iprintf("Touch y = %04X, %04X\n", touchXY.y, touchXY.py);

		// next flash movie frame
		cmd = FLASH_WAKEUP;
		wakeUp = FlashExec(flashHandle, cmd, 0, &wd);


		// this function can be used to switch buffers
		//FlashSetBuffer(flashHandle, (unsigned char *) framebuf);


		swiWaitForVBlank();
		DC_FlushRange(framebuf, 256*192*2);
		dmaCopy(framebuf, VRAM, 256*192*2);

	}

	return 0;
}
